from django.urls import path
from .views import verify_user_and_activate, request_new_link

urlpatterns = [
    path(f'user/verify-email/<useremail>/<usertoken>/', verify_user_and_activate, name='verify-email'),
    path(f'user/verify-email/request-new-link/<useremail>/<usertoken>/', request_new_link, name='request-new-link'),
]



# Generated by Django 3.0.8 on 2021-04-24 09:56

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='LinkCounter',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sent_count', models.IntegerField()),
                ('requester', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from django.conf import settings


class GetFieldFromSettings:
    """
    This class fetches the attributes that are defined in settings.py of your project by user OR Django itself.
    self.default_configs : is a dict with keys as the names used in this app and values being a tuple of 
                           attributes defined in settings.py and their corresponding default values if not found.
    
    There is a special case in "get" method, if you set "VERIFICATION_SUCCESS_TEMPLATE" as None is settings.py, it 
    will skip the intermidiate page where success information is displayed. (This is better explained in docs.)

    The "get" method takes the name of the attributes as input, checks for it in settings.py, 
            if found:
                returns the corresponding value.
            else:
                returns the default value from "self.defaults_configs".
    """

    def __init__(self):

        self.defaults_configs = {
            'debug_settings': (
                'DEBUG', 
                False
            ),

            'subject': (
                "SUBJECT", 
                "Email Verification Mail"
            ),

            'email_field_name': (
                "EMAIL_FIELD_NAME", 
                "email",
            ),

            'html_message_template': (
                "HTML_MESSAGE_TEMPLATE", 
                'verify_email/email_verification_msg.html'
            ),

            'from_alias': (
                "DEFAULT_FROM_EMAIL", 
                'noreply<noreply@gmail.com>',
            ),

            'verification_success_redirect': (
                'LOGIN_URL',
                'accounts_login'
            ),

            'verification_success_template': (
                'VERIFICATION_SUCCESS_TEMPLATE',
                'verify_email/email_verification_successful.html'
            ),

            'verification_success_msg': (
                'VERIFICATION_SUCCESS_MSG',
                "Your Email is verified successfully and account has been activated. "
                "You can login with the credentials now..."
            ),

            'verification_failed_template': (
                'VERIFICATION_FAILED_TEMPLATE',
                'verify_email/email_verification_failed.html'
            ),

            'link_expired_template': (
                'LINK_EXPIRED_TEMPLATE',
                'verify_email/link_expired.html'
            ),

            'verification_failed_msg': (
                'VERIFICATION_FAILED_MSG',
                "There is something wrong with this link, can't verify the user..."
            ),

            'salt': (
                'HASH_SALT',
                None
            ),

            'sep': (
                'SEPARATOR',
                ':'
            ),

            'key': (
                'HASHING_KEY',
                None
            ),

            'max_age': (
                'EXPIRE_AFTER',
                None
            ),

            'max_retries': (
                'MAX_RETRIES',
                2
            )
        }

    def get(self, field_name, raise_exception=True, default_type=str):
        attr = getattr(
            settings, 
            self.defaults_configs[field_name][0],  # get field from settings
            self.defaults_configs[field_name][1]   # get default value if field not defined
        )
        if (attr == '' or attr is None or not isinstance(field_name, default_type)) and raise_exception:
            if field_name == 'verification_success_template' and attr is None:
                return None
            raise AttributeError
        return attr


        
            
        
            

































from django.db.models.signals import post_save
from django.dispatch import receiver
from .models import LinkCounter, USER


@receiver(post_save, sender=USER)
def increase_count(sender, instance, created, **kwargs):
    if created:
        LinkCounter.objects.create(requester=instance, sent_count=1)


@receiver(post_save, sender=USER)
def save_count(sender, instance, **kwargs):
    try:
        instance.linkcounter.save()
    except Exception as err:
        print(err)



from django.db import models
from django.contrib.auth import get_user_model

USER = get_user_model()


class LinkCounter(models.Model):
    requester = models.OneToOneField(USER, on_delete=models.CASCADE)
    sent_count = models.IntegerField()

    def __str__(self):
        return self.requester.username



from django.utils import timezone
from .token_manager import TokenManager, error_type, MaxRetriesExceeded
from django.core.signing import SignatureExpired, BadSignature


class _UserActivationProcess:
    """
    This class is pretty self.explanatory...
    """

    def __init__(self):
        self.token_manager = TokenManager()

    @staticmethod
    def __activate_user(user):
        user.is_active = True
        user.last_login = timezone.now()
        user.save()

    def verify_token(self, useremail, usertoken):
        try:
            user = self.token_manager.decrypt_link(useremail, usertoken)
            if user:
                self.__activate_user(user)
                return True
            return False
        except (ValueError, TypeError):
            return error_type.failed
        except SignatureExpired:
            return error_type.expired
        except BadSignature:
            return error_type.tempered
        except MaxRetriesExceeded:
            return error_type.mre


def _verify_user(useremail, usertoken):
    return _UserActivationProcess().verify_token(useremail, usertoken)



from .app_configurations import GetFieldFromSettings
from .confirm import _verify_user
from django.contrib import messages
from django.urls import reverse
from django.shortcuts import render, redirect

from .token_manager import error_type
from .email_handler import resend_verification_email

success_redirect = GetFieldFromSettings().get('verification_success_redirect')

success_msg = GetFieldFromSettings().get('verification_success_msg')
failed_msg = GetFieldFromSettings().get('verification_failed_msg')

failed_template = GetFieldFromSettings().get('verification_failed_template')
success_template = GetFieldFromSettings().get('verification_success_template')
link_expired_template = GetFieldFromSettings().get('link_expired_template')


def verify_user_and_activate(request, useremail, usertoken):
    """
    A view function already implemented for you so you don't have to implement any function for verification
    as this function will be automatically be called when user clicks on verification link.

    verify the user's email and token and redirect'em accordingly.
    """

    verified = _verify_user(useremail, usertoken)
    if verified is True:
        if success_redirect and not success_template:
            messages.success(request, success_msg)
            return redirect(to=success_redirect)
        return render(
            request,
            template_name=success_template,
            context={
                'msg': success_msg,
                'status': 'Verification Successful!',
                'link': reverse(success_redirect)
            }
        )
    elif verified == error_type.expired:
        return render(
            request,
            template_name=link_expired_template,
            context={
                'msg': 'The link has lived its life :( Request a new one!',
                'status': 'Expired!',
                'encoded_email': useremail,
                'encoded_token': usertoken
            }
        )
    elif verified == error_type.tempered:
        return render(
            request,
            template_name=failed_template,
            context={
                'msg': 'This link was modified before verification.',
                'minor_msg': 'Cannot request another verification link with faulty link.',
                'status': 'Faulty Link Detected!',
            }
        )
    elif verified == error_type.mre:
        return render(
            request,
            template_name=failed_template,
            context={
                'msg': 'You have exceeded the maximum verification requests! Contact admin.',
                'status': 'Maxed out!',
            }
        )
    else:
        return render(
            request,
            template_name=failed_template,
            context={
                'msg': failed_msg,
                'minor_msg': 'There is something wrong with this link...',
                'status': 'Verification Failed!',
            }
        )


def request_new_link(request, useremail, usertoken):
    status = resend_verification_email(request, useremail, usertoken)
    if status:
        return render(
            request,
            template_name='verify_email/display_message.html',
            context={
                'msg': "You have requested another verification email!",
                'minor_msg': 'Your verification link has been sent',
                'status': 'Email Sent!',
            }
        )
    elif status == error_type.mre:
        return render(
            request,
            template_name=failed_template,
            context={
                'msg': 'You have exceeded the maximum verification requests! Contact admin.',
                'status': 'Maxed out!',
            }
        )
    else:
        return render(
            request,
            template_name=failed_template,
            context={
                'msg': failed_msg,
                'minor_msg': 'Cannot send verification link to you! contact admin.',
                'status': 'Verification Failed!',
            }
        )



from django.core.mail import BadHeaderError, send_mail
from django.template.loader import render_to_string
from django.utils.html import strip_tags
from smtplib import SMTPException
from .app_configurations import GetFieldFromSettings

from .token_manager import TokenManager


class _VerifyEmail:
    """
    This class does two things:
    1. set each user as inactive and saves it
    2. sends the email to user with that link.
    """

    def __init__(self):
        self.settings = GetFieldFromSettings()
        self.token_manager = TokenManager()

    # Private :
    def __send_email(self, msg, useremail):
        subject = self.settings.get('subject')
        try:
            send_mail(subject, strip_tags(msg), from_email=self.settings.get('from_alias'),
                      recipient_list=[useremail], html_message=msg)
            return True
        except (BadHeaderError, SMTPException):
            return False

    # Public :
    def send_verification_link(self, request, form):
        inactive_user = form.save(commit=False)
        inactive_user.is_active = False
        inactive_user.save()

        try:
            useremail = form.cleaned_data.get(self.settings.get('email_field_name'))
            if not useremail:
                raise KeyError(
                    'No key named "email" in your form. Your field should be named as email in form OR set a variable'
                    ' "EMAIL_FIELD_NAME" with the name of current field in settings.py if you want to use current name '
                    'as email field.'
                )

            verification_url = self.token_manager.generate_link(request, inactive_user, useremail)
            msg = render_to_string(self.settings.get('html_message_template', raise_exception=True),
                                   {"link": verification_url}, request=request)

            if self.__send_email(msg, useremail):
                return inactive_user
            inactive_user.delete()
            return False

        except Exception as error:

            inactive_user.delete()
            if self.settings.get('debug_settings'):
                raise Exception(error)

    def resend_verification_link(self, request, encoded_email, encoded_token):
        """
        This method needs the previously sent link to get encoded email and token from that.
        """
        assert encoded_email is not None
        assert encoded_token is not None
        link = self.token_manager.request_new_link(request, encoded_email, encoded_token)
        email = self.token_manager.perform_decoding(encoded_email)
        if link:
            msg = render_to_string(self.settings.get('html_message_template', raise_exception=True),
                                   {"link": link}, request=request)
            try:
                self.__send_email(msg, email)
                return True
            except (BadHeaderError, SMTPException):
                return False
        else:
            return link


#  These is supposed to be called outside of this module
def send_verification_email(request, form):
    return _VerifyEmail().send_verification_link(request, form)


#  These is supposed to be called outside of this module
def resend_verification_email(request, encoded_email, encoded_token):
    return _VerifyEmail().resend_verification_link(request, encoded_email, encoded_token)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    long_description=long_description,
    long_description_content_type="text/markdown",    
)
